{******************************************************************************}
{**}
{**  Project Name:	DropInfo}
{**     File Name:	DIStuff.p}
{**}
{**   Description:	Contains the DropInfo-specific code}
{**}
{*******************************************************************************}
{**                       A U T H O R   I D E N T I T Y}
{*******************************************************************************}
{**}
{**	Initials	Name}
{**	--------	-----------------------------------------------}
{**	SCS			Stephan Somogyi}
{**	LDR			Leonard Rosenthol}
{**}
{*******************************************************************************}
{**                      R E V I S I O N   H I S T O R Y}
{*******************************************************************************}
{**}
{**	  Date		Vers	Author	Description}
{**	--------	-----	------	---------------------------------------------}
{**	12/22/91			LDR		Finished up Pascal port}
{**	12/11/91			LDR		Ported to Pascal}
{**}
{**	06.09.91	a(-1)	SCS		files implemented except for resmap locking}
{**	10.09.91	a2				removed resmap locking as there is no nice way to do it}
{**								added folders, not sure they work the way they should, though}
{**	25.09.91	a3				added the laurie gill feature: cmd-q quits regardless of whether}
{**								there are any files/folders left to deal with. added flushvols.}
{**	26.09.91	a4				added change all. fixed some stupid a3 bugs.}
{**	27.09.91	b1				added dimming of change all if only one file dropped. source}
{**								reality check.}
{**	01.10.91	b1				changed oapp behavior to what it should be. better volume supt.}
{**	09.10.91	fc1				moved initial selection to type rather than creator}
{**				fc2				minor divergence where SCS and LDR both did similar things}
{**	29.11.91	fc3				used 11/24/91 DropShell}
{**								made non-inline MyBitTst and MyBitSetClr}
{**								made DI compilable by both THINK and MPW}
{**	30.11.91	fc3+	LDR		made MUCH more use of enum's & #defines rather than hard coded values}
{**								Also added some comments}
{**								Applied hack in ItsADir for MPW C compiler}
{**	01.12.91	F4		SCS		Reformatting and commenting}
{**								Replaced IUEqualString with EqualString for better case sensitivity}
{**}
{******************************************************************************}

UNIT DIStuff;
INTERFACE

{$IFC THINK_Pascal}
	USES
		Aliases, AppleEvents, PPCToolbox, Processes, Script, DSGlobals, DSUtils, DIUtils;	{just the DropShell files}
{$ELSEC}
	USES
	{ First load standard interface files}
		MemTypes, QuickDraw, 

	{ Now include the stuff from OSIntf }
		OSIntf, 

	{ Now Include the stuff from ToolIntf.p }
		ToolIntf, Packages, GestaltEqu, 

	{ Then any OTHER Toolbox interfaces... }
		Files, Aliases, AppleEvents, PPCToolbox, Processes, 

	{ And finally any files from DropShell }
		DSGlobals, DSUtils, DIUtils;
{$ENDC THINK_Pascal}

	CONST
	{bit numbers for the relevant flags in the FInfo struct}
		bIsLocked = 0;
		bIsDir = 4;
		bIsShared = 6;
		bHasNoINITs = 7;
		bInited = 8;
		bCustomIcn = 10;
		bStationary = 11;
		bNameLock = 12;
		bBundle = 13;
		bInvisible = 14;
		bIsAlias = 15;

	TYPE
		bitsArray = ARRAY[bIsLocked..bIsAlias] OF Boolean;

	VAR
		fileBits: bitsArray;		{ contains the various bits states }
		gAbort: Boolean;			{ quit and ignore others? }
		gChgAll: Boolean;			{ used to track change all }
		gIsFiles: Boolean;			{ Files or folders? }
		gWarned: Boolean;			{ has the user been warned? }
		gTypeStr, gCreaStr: Str255;	{ used by ChangeAll to reset the type/creator }
		gTempStr: Str255;			{ global tempStr }
		gOnlyOne: Boolean;			{ only one file or folder? }

{---------------------}
{Interface Definitions}
{---------------------}
	PROCEDURE GetTheInfo (pFSS: FSSpecPtr);


IMPLEMENTATION


{Does a bit test in 68k order, not the reverse like the Toolbox's BitTst() }
	FUNCTION MyBitTst (pBMap, pBitNum: integer): Boolean;
	BEGIN
		MyBitTst := BTst(Longint(pBMap), pBitNum);
	END;

{ Clears or sets the specified bit depending on pState }
	PROCEDURE MyBitSetClr (VAR pBMap: Integer; pBitNum: Integer; pState: Boolean);
		VAR
			tempLong: longint;
	BEGIN
		tempLong := pBMap;		{we need to do this since ThP won't coerce VAR params}

		IF (pState) THEN
			BSet(tempLong, pBitNum)
		ELSE
			BClr(tempLong, pBitNum);

		pBMap := LoWord(tempLong);
	END;

{ toggles a Boolean and sets the dirty flag as well }
	PROCEDURE ToggleBit (VAR pBool: Boolean);
	BEGIN
		pBool := NOT pBool;
		fileBits[0] := TRUE;
	END;


{}
{	Takes a file's or folder's information and puts it in the }
{	right places in DI's array}
{}
	PROCEDURE InfoIn (pFSflags: integer; pCBlk: CInfoPBPtr);
		VAR
			lCtr: integer;
	BEGIN
		IF (NOT myBitTst(pCBlk^.ioFlAttrib, bIsDir)) THEN
		BEGIN	{not a directory}
			FOR lCtr := bIsShared TO bIsAlias DO
				fileBits[lCtr] := myBitTst(pFSFlags, lCtr);

			fileBits[1] := myBitTst(pCBlk^.ioFlAttrib, bIsLocked);	{file locked state}
			fileBits[0] := FALSE;	{clear the dirty flag}
		END
		ELSE	{it's a file!}
		BEGIN
			fileBits[7] := myBitTst(pCBlk^.ioFlFndrInfo.fdFlags, bInvisible);	{invisible}
			fileBits[6] := myBitTst(pCBlk^.ioFlFndrInfo.fdFlags, bNameLock);	{name lock}
			fileBits[5] := myBitTst(pCBlk^.ioFlFndrInfo.fdFlags, bCustomIcn);	{custom icon}
			fileBits[4] := myBitTst(pCBlk^.ioFlFndrInfo.fdFlags, bInited);		{initted}
			fileBits[0] := FALSE;	{clear the dirty flag}
		END;
	END;


{}
{	The reverse of InfoIn; takes the file's or folder's information from }
{	DI's array and fills out the OS' data structure}
{}
	PROCEDURE InfoOut (VAR pFSflags: integer; pFSS: FSSpec; pCBlk: CInfoPBPtr);
		VAR
			lErr: OSErr;
			lCtr: integer;
	BEGIN
		IF (NOT myBitTst(pCBlk^.ioFlAttrib, bIsDir)) THEN
		BEGIN	{not a directory}
			FOR lCtr := bIsShared TO bIsAlias DO
				myBitSetClr(pFSFlags, lCtr, fileBits[lCtr]);

			IF (fileBits[1]) THEN	{deal with file lock}
				lErr := FSpSetFLock(pFSS)
			ELSE
				lErr := FSpRstFLock(pFSS);
		END
		ELSE	{it's a file!}
		BEGIN
			myBitSetClr(pFSFlags, bInvisible, fileBits[7]);
			myBitSetClr(pFSFlags, bNameLock, fileBits[6]);
			myBitSetClr(pFSFlags, bCustomIcn, fileBits[5]);
			myBitSetClr(pFSFlags, bInited, fileBits[4]);
		END;
	END;

{}
{	ModalDialog filterProc }
{	In addition to the standard key support, it also supports Cmd-Q as cancel}
{}
	FUNCTION MyStdKeyFilter (pDial: DialogPtr; VAR pEventRec: EventRecord; VAR pItemHit: integer): Boolean;
		CONST		{need these since Think Pascal doesn't like CHR()'s in CASE statements}
			enter = CHR(3);
			return = CHR(13);
			esc = CHR(27);
		VAR
			lKey: char;
			retVal: Boolean;
	BEGIN
		retVal := FALSE;	{assume no intervention}

		IF ((pEventRec.what = keyDown) OR (pEventRec.what = autoKey)) THEN
		BEGIN
			lKey := Char(BAND(pEventRec.message, $FF));
			CASE (lKey) OF
				return, enter: 
				BEGIN
					pItemHit := 1;
					FlashItem(pDial, pItemHit);
					retVal := TRUE;
				END;

				esc: 
				BEGIN
					pItemHit := 2;
					FlashItem(pDial, pItemHit);
					retVal := TRUE;
				END;

				'.', '>', 'q', 'Q':	{cmd-.}
					IF (BAND(pEventRec.modifiers, cmdKey) > 0) THEN
					BEGIN
						pItemHit := 2;
						FlashItem(pDial, pItemHit);
						retVal := TRUE;

						IF ((lKey = 'q') OR (lKey = 'Q')) THEN
							gAbort := TRUE;
					END;
			END;
		END;

		MyStdKeyFilter := retVal;
	END;


{}
{	Routine called if the thing in question is a file. }
{	}
{	This actually does the work }
{}
	PROCEDURE ItsAFile (pFSS: FSSpec; pCBlk: CInfoPBPtr);
		VAR
			lErr: OSErr;
			lFInfo: FInfo;
			lNameStr: Str255;
			lDial: DialogPtr;
			lCtr, lItemHit: integer;

	BEGIN
		lErr := FSpGetFInfo(pFSS, lFinfo);

		IF (NOT gChgAll) THEN
		BEGIN
			InfoIn(lFinfo.fdFlags, @pCBlk);

			lDial := GetNewDialog(130, NIL, Pointer(-1));
			DefBut(lDial);

		{	}
{			Set up the dialog}
{		}
{			Probably should consider NOT using BlockMove}
{			as it is VERY inefficient on an '040 due to CacheFlush}
{		}
			BlockMove(@pFSS.name, @lNameStr, ORD(pFSS.name[0]) + 1);
			gTypeStr[0] := CHR(4);
			BlockMove(@lFinfo.fdType, @gTypeStr[1], 4);
			gCreaStr[0] := CHR(4);
			BlockMove(@lFinfo.fdCreator, @gCreaStr[1], 4);
			SetTxtItem(lDial, 5, lNameStr);
			SetTxtItem(lDial, 3, gTypeStr);
			SetTxtItem(lDial, 4, gCreaStr);


			FOR lCtr := bIsShared TO bInited DO
				CheckBox(lDial, lCtr, fileBits[lCtr]);
			FOR lCtr := bCustomIcn TO bIsAlias DO
				CheckBox(lDial, lCtr, fileBits[lCtr]);

			CheckBox(lDial, 16, fileBits[1]);							{ locked }

			IF (gOnlyOne) THEN
				DisEnAble(lDial, 18, FALSE);

			DrawDialog(lDial);
			SelIText(lDial, 3, 0, 32767);

			REPEAT
				gAbort := FALSE;
				ModalDialog(@MyStdKeyFilter, lItemHit);
				CASE (lItemHit) OF
					1: 
						;	{OK}
						{Do nothing}
					2:	{Cancel}
						fileBits[0] := FALSE;	{clear the dirty flag}
					16:	{locked state}
					BEGIN
						ToggleBit(fileBits[1]);
						CheckBox(lDial, 16, fileBits[1]);
					END;
					18:	{change all}
					BEGIN
						gChgAll := TRUE;
						gIsFiles := TRUE;
					END;
					OTHERWISE	{everything else!}
					BEGIN
						ToggleBit(fileBits[lItemHit]);
						CheckBox(lDial, lItemHit, fileBits[lItemHit]);
					END;
				END;
			UNTIL ((lItemHit = 1) OR (lItemHit = 2) OR (lItemHit = 18));

			HideWindow(lDial);

			IF ((lItemHit <> 2) AND (NOT gAbort)) THEN	{if not cancel or abort}
			BEGIN
				GetTxtItem(lDial, 3, gTempStr);

				IF (NOT EqualString(gTempStr, gTypeStr, TRUE, TRUE)) THEN
				BEGIN
				{}
{					Probably should consider NOT using BlockMove}
{					as it is VERY inefficient on an '040 due to CacheFlush}
{				}
					BlockMove(@gTempStr[1], @lFinfo.fdType, 4);
					BlockMove(@gTempStr[1], @gTypeStr[1], 4);
					fileBits[0] := TRUE;
				END;

				GetTxtItem(lDial, 4, gTempStr);
				IF (NOT EqualString(gTempStr, gCreaStr, TRUE, TRUE)) THEN
				BEGIN
				{}
{					Probably should consider NOT using BlockMove}
{					as it is VERY inefficient on an '040 due to CacheFlush}
{				}
					BlockMove(@gTempStr[1], @lFinfo.fdCreator, 4);
					BlockMove(@gTempStr[1], @gCreaStr[1], 4);
					fileBits[0] := TRUE;
				END;
				GetTxtItem(lDial, 5, gTempStr);
				IF (NOT EqualString(gTempStr, lNameStr, TRUE, TRUE)) THEN
				BEGIN
					IF (ORD(gTempStr[0]) > 32) THEN
						gTempStr[0] := CHR(32);
					lErr := FSpRstFLock(pFSS);				{will be reset later if necessary}
					lErr := FSpRename(pFSS, gTempStr);

					IF (lErr = -48) THEN
					BEGIN
						lErr := Alert(132, NIL);
						fileBits[0] := FALSE;
					END
					ELSE
					BEGIN
						BlockMove(@gTempStr, @pFSS.name, ORD(gTempStr[0]) + 1);
						fileBits[0] := TRUE;
					END;
				END;
			END;

			DisposDialog(lDial);
		END
		ELSE
		BEGIN
			fileBits[0] := TRUE;
		{}
{			Probably should consider NOT using BlockMove}
{			as it is VERY inefficient on an '040 due to CacheFlush}
{		}
			BlockMove(@gTypeStr[1], @lFinfo.fdType, 4);
			BlockMove(@gCreaStr[1], @lFinfo.fdCreator, 4);
		END;

		IF (fileBits[0]) THEN		{dirty flag is set}
		BEGIN
			InfoOut(lFinfo.fdFlags, pFSS, pCBlk);

			pCBlk^.ioDirID := pFSS.parID;
			IF (gChgAll) THEN
				pCBlk^.ioNamePtr := NIL
			ELSE
				pCBlk^.ioNamePtr := @gTempStr;
			lErr := PBSetCatInfo(pCBlk, FALSE);

			lErr := FSpSetFInfo(pFSS, lFinfo);
		END;

		IF (gAbort) THEN				{user wants to quit}
			SendQuitToSelf;				{send a quit event to myself}
	END;


{}
{	Routine called if the thing in question is a folder. }
{	}
{	This actually does the work}
{}
	PROCEDURE ItsADir (pFSS: FSSpec; pCBlk: CInfoPBPtr);
		VAR
			lErr: OSErr;
			lFInfo: FInfo;
			lHBlk: HParamBlockRec;
			lNameStr: Str255;
			lDial: DialogPtr;
			lCtr: integer;
			lItemHit: integer;

	BEGIN
		lErr := noErr;

		IF (NOT gChgAll) THEN
		BEGIN
			InfoIn(lFinfo.fdFlags, pCBlk);

			lDial := GetNewDialog(129, NIL, Pointer(-1));
			DefBut(lDial);

		{set up the dialog}

			BlockMove(@pFSS.name, @lNameStr, ORD(pFSS.name[0]) + 1);
			SetTxtItem(lDial, 3, lNameStr);

			FOR lCtr := 4 TO 7 DO
				CheckBox(lDial, lCtr, fileBits[lCtr]);

			IF (gOnlyOne) THEN
				DisEnAble(lDial, 8, FALSE);

			DrawDialog(lDial);
			SelIText(lDial, 3, 0, 32767);

			REPEAT
				ModalDialog(@MyStdKeyFilter, lItemHit);
				CASE (lItemHit) OF
					1: 
						;			{OK}
						{do nothing}
					2:			{Cancel}
						fileBits[0] := FALSE;	{clear dirty flag}
					8:			{Change All}
					BEGIN
						gChgAll := TRUE;
						gIsFiles := FALSE;
					END;
					OTHERWISE	{all the bits}
					BEGIN
						ToggleBit(fileBits[lItemHit]);
						CheckBox(lDial, lItemHit, fileBits[lItemHit]);
					END;
				END;
			UNTIL ((lItemHit = 1) OR (lItemHit = 2) OR (lItemHit = 8));

			HideWindow(lDial);

			IF (gAbort) THEN	{cmd q}
			BEGIN
				DisposDialog(lDial);
				SendQuitToSelf;
			END;

			IF (lItemHit <> 2) THEN	{if not cancel}
			BEGIN
				GetTxtItem(lDial, 3, gTempStr);

				IF (NOT EqualString(gTempStr, lNameStr, TRUE, TRUE)) THEN
				BEGIN
					IF (ORD(gTempStr[0]) > 32) THEN
						gTempStr[0] := CHR(32);

					lHBlk.ioNamePtr := @lNameStr;
					lHBlk.ioVRefNum := pFSS.vRefNum;
					lHBlk.ioMisc := Ptr(@gTempStr);
					lHBlk.ioDirID := pFSS.parID;
					lErr := PBHRename(@lHBlk, FALSE);		{rename the dir}

					IF (lErr = -48) THEN
					BEGIN
						lErr := Alert(132, NIL);
						fileBits[0] := FALSE;
					END
					ELSE
						fileBits[0] := TRUE;

				END;
			END;
			DisposDialog(lDial);
		END
		ELSE
			fileBits[0] := true;

		IF (fileBits[0]) THEN	{dirty flag is set}
		BEGIN
			InfoOut(pCBlk^.ioFlFndrInfo.fdFlags, pFSS, pCBlk);

			pCBlk^.ioDirID := pFSS.parID;
			IF (NOT gChgAll) THEN
				pCBlk^.ioNamePtr := @gTempStr;
			lErr := PBSetCatInfo(pCBlk, FALSE);
		END;
	END;


{}
{	This is called by DropShell, once for every FSSpec received via odoc. GetTheInfo}
{	is simply a dispatcher to ItsAFile and ItsAFolder. Once the alterations have been made}
{	it also touched the parent dir's modification date, thereby causing the Finder to update}
{	that directory's window.}
{}
	PROCEDURE GetTheInfo (pFSS: FSSpecPtr);
		VAR
			lErr: OSErr;
			lCBlk: CInfoPBRec;
	BEGIN
		lCBlk.ioVRefNum := pFSS^.vRefNum;
		lCBlk.ioNamePtr := @pFSS^.name;
		lCBlk.ioDirID := pFSS^.parID;
		lCBlk.ioFDirIndex := 0;
		lCBlk.ioCompletion := NIL;

		lErr := PBGetCatInfo(@lCBlk, FALSE);

		IF (NOT MyBitTst(lCBlk.ioFlAttrib, bIsDir)) THEN
		BEGIN
			IF (gChgAll AND (NOT gIsFiles)) THEN
			BEGIN
				IF (NOT gWarned) THEN
				BEGIN
					ParamText('folders', '', '', '');	{should be in a resource!}
					lErr := Alert(133, NIL);
					gWarned := TRUE;
				END;
				EXIT(GetTheInfo);
			END
			ELSE
				ItsAFile(pFSS^, @lCBlk);
		END
		ELSE
		BEGIN
			IF (gChgAll AND gIsFiles) THEN
			BEGIN
				IF (NOT gWarned) THEN
				BEGIN
					ParamText('files', '', '', '');	{should be in a resource!}
					lErr := Alert(133, NIL);
					gWarned := TRUE;
				END;
				EXIT(GetTheInfo);
			END
			ELSE
				ItsADir(pFSS^, @lCBlk);
		END;

	{	}
{		touch the parent dir's mod date	}
{		We need to do this to get the Finder to update itself on demand!}
{	}

		IF (fileBits[0]) THEN
		BEGIN
			IF (pFSS^.parID <> 1) THEN				{if it's a vol then reuse the NameStr}
				lCBlk.ioNamePtr := NIL;
			lCBlk.ioVRefNum := pFSS^.vRefNum;
			lCBlk.ioDrDirID := pFSS^.parID;
			lCBlk.ioFDirIndex := 0;
			lCBlk.ioCompletion := NIL;

			lErr := PBGetCatInfo(@lCBlk, FALSE);
			GetDateTime(lCBlk.ioDrMdDat);
			lCBlk.ioDrDirID := pFSS^.parID;
			lErr := PBSetCatInfo(@lCBlk, FALSE);

			lErr := FlushVol(NIL, pFSS^.vRefNum);
		END;
	END;

END.